@@ -8,11 +8,57 @@ License: No restrictions, use this as you need.
 
 import argparse
 import json
+import os
 import sys
+import subprocess
+
+try:
+    import pathlib
+except ImportError:
+    message = "Failed to load pathlib. Upgrade python to 3.x or install pathlib with 'pip install pathlib'."
+    error_id = 'pythonError'
+    if '--cli' in sys.argv:
+        msg = { 'file': '',
+                'linenr': 0,
+                'column': 0,
+                'severity': 'error',
+                'message': message,
+                'addon': 'cppcheckdata',
+                'errorId': error_id,
+                'extra': ''}
+        sys.stdout.write(json.dumps(msg) + '\n')
+    else:
+        sys.stderr.write('%s [%s]\n' % (message, error_id))
+    sys.exit(1)
 
 from xml.etree import ElementTree
 from fnmatch import fnmatch
 
+EXIT_CODE = 0
+
+current_dumpfile_suppressions = []
+
+def _load_location(location, element):
+    """Load location from element/dict"""
+    location.file = element.get('file')
+    line = element.get('line')
+    if line is None:
+        line = element.get('linenr')
+    if line is None:
+        line = '0'
+    location.linenr = int(line)
+    location.column = int(element.get('column', '0'))
+
+
+class Location:
+    """Utility location class"""
+    file = None
+    linenr = None
+    column = None
+    def __init__(self, element):
+        _load_location(self, element)
+
+
 class Directive:
     """
     Directive class. Contains information about each preprocessor directive in the source code.
@@ -34,12 +80,11 @@ class Directive:
     str = None
     file = None
     linenr = None
-    column = 0
+    column = None
 
     def __init__(self, element):
         self.str = element.get('str')
-        self.file = element.get('file')
-        self.linenr = int(element.get('linenr'))
+        _load_location(self, element)
 
     def __repr__(self):
         attrs = ["str", "file", "linenr"]
@@ -48,6 +93,57 @@ class Directive:
             ", ".join(("{}={}".format(a, repr(getattr(self, a))) for a in attrs))
         )
 
+class MacroUsage:
+    """
+    Tracks preprocessor macro usage
+    """
+
+    name = None  # Macro name
+    file = None
+    linenr = None
+    column = None
+    usefile = None
+    uselinenr = None
+    usecolumn = None
+
+    def __init__(self, element):
+        self.name = element.get('name')
+        _load_location(self, element)
+        self.usefile = element.get('usefile')
+        self.useline = element.get('useline')
+        self.usecolumn = element.get('usecolumn')
+
+    def __repr__(self):
+        attrs = ["name", "file", "linenr", "column", "usefile", "useline", "usecolumn"]
+        return "{}({})".format(
+            "MacroUsage",
+            ", ".join(("{}={}".format(a, repr(getattr(self, a))) for a in attrs))
+        )
+
+
+class PreprocessorIfCondition:
+    """
+    Information about #if/#elif conditions
+    """
+
+    file = None
+    linenr = None
+    column = None
+    E = None
+    result = None
+
+    def __init__(self, element):
+        _load_location(self, element)
+        self.E = element.get('E')
+        self.result = int(element.get('result'))
+
+    def __repr__(self):
+        attrs = ["file", "linenr", "column", "E", "result"]
+        return "{}({})".format(
+            "PreprocessorIfCondition",
+            ", ".join(("{}={}".format(a, repr(getattr(self, a))) for a in attrs))
+        )
+
 
 class ValueType:
     """
@@ -66,25 +162,15 @@ class ValueType:
     def __init__(self, element):
         self.type = element.get('valueType-type')
         self.sign = element.get('valueType-sign')
-        bits = element.get('valueType-bits')
-        if bits:
-            self.bits = int(bits)
+        self.bits = int(element.get('valueType-bits', 0))
         self.typeScopeId = element.get('valueType-typeScope')
         self.originalTypeName = element.get('valueType-originalTypeName')
-        constness = element.get('valueType-constness')
-        if constness:
-            self.constness = int(constness)
-        else:
-            self.constness = 0
-        pointer = element.get('valueType-pointer')
-        if pointer:
-            self.pointer = int(pointer)
-        else:
-            self.pointer = 0
+        self.constness = int(element.get('valueType-constness', 0))
+        self.pointer = int(element.get('valueType-pointer', 0))
 
     def __repr__(self):
         attrs = ["type", "sign", "bits", "typeScopeId", "originalTypeName",
-                 "constness", "constness", "pointer"]
+                 "constness", "pointer"]
         return "{}({})".format(
             "ValueType",
             ", ".join(("{}={}".format(a, repr(getattr(self, a))) for a in attrs))
@@ -110,7 +196,7 @@ class Token:
 
     The CppcheckData.tokenlist is a list of Token items
 
-    C++ class: http://cppcheck.net/devinfo/doxyoutput/classToken.html
+    C++ class: https://cppcheck.sourceforge.io/devinfo/doxyoutput/classToken.html
 
     Attributes:
         str                Token string
@@ -123,7 +209,7 @@ class Token:
         isName             Is this token a symbol name
         isNumber           Is this token a number, for example 123, 12.34
         isInt              Is this token a int value such as 1234
-        isFloat            Is this token a int value such as 12.34
+        isFloat            Is this token a float value such as 12.34
         isString           Is this token a string literal such as "hello"
         strlen             string length for string literal
         isChar             Is this token a char literal such as 'x'
@@ -135,11 +221,16 @@ class Token:
         isUnsigned         Is this token a unsigned type
         isSigned           Is this token a signed type
         isExpandedMacro    Is this token a expanded macro token
+        isRemovedVoidParameter  Has void parameter been removed?
+        isSplittedVarDeclComma  Is this a comma changed to semicolon in a splitted variable declaration ('int a,b;' => 'int a; int b;')
+        isSplittedVarDeclEq     Is this a '=' changed to semicolon in a splitted variable declaration ('int a=5;' => 'int a; a=5;')
+        isImplicitInt      Is this token an implicit "int"?
         varId              varId for token, each variable has a unique non-zero id
         variable           Variable information for this token. See the Variable class.
         function           If this token points at a function call, this attribute has the Function
                            information. See the Function class.
-        values             Possible values of token
+        values             Possible/Known values of token
+        impossible_values  Impossible values of token
         valueType          type information
         typeScope          type scope (token->type()->classScope)
         astParent          ast parent
@@ -183,6 +274,10 @@ class Token:
     isUnsigned = False
     isSigned = False
     isExpandedMacro = False
+    isRemovedVoidParameter = False
+    isSplittedVarDeclComma = False
+    isSplittedVarDeclEq = False
+    isImplicitInt = False
     varId = None
     variableId = None
     variable = None
@@ -190,6 +285,7 @@ class Token:
     function = None
     valuesId = None
     values = None
+    impossible_values = None
     valueType = None
 
     typeScopeId = None
@@ -243,6 +339,14 @@ class Token:
                 self.isLogicalOp = True
         if element.get('isExpandedMacro'):
             self.isExpandedMacro = True
+        if element.get('isRemovedVoidParameter'):
+            self.isRemovedVoidParameter = True
+        if element.get('isSplittedVarDeclComma'):
+            self.isSplittedVarDeclComma = True
+        if element.get('isSplittedVarDeclEq'):
+            self.isSplittedVarDeclEq = True
+        if element.get('isImplicitInt'):
+            self.isImplicitInt = True
         self.linkId = element.get('link')
         self.link = None
         if element.get('varId'):
@@ -265,15 +369,14 @@ class Token:
         self.astOperand1 = None
         self.astOperand2Id = element.get('astOperand2')
         self.astOperand2 = None
-        self.file = element.get('file')
-        self.linenr = int(element.get('linenr'))
-        self.column = int(element.get('column'))
+        _load_location(self, element)
 
     def __repr__(self):
         attrs = ["Id", "str", "scopeId", "isName", "isUnsigned", "isSigned",
                 "isNumber", "isInt", "isFloat", "isString", "strlen",
                 "isChar", "isOp", "isArithmeticalOp", "isComparisonOp",
-                "isLogicalOp", "isExpandedMacro", "linkId", "varId",
+                "isLogicalOp", "isExpandedMacro", "isSplittedVarDeclComma",
+                "isSplittedVarDeclEq", "isImplicitInt", "linkId", "varId",
                 "variableId", "functionId", "valuesId", "valueType",
                 "typeScopeId", "astParentId", "astOperand1Id", "file",
                 "linenr", "column"]
@@ -287,7 +390,15 @@ class Token:
         self.link = IdMap[self.linkId]
         self.variable = IdMap[self.variableId]
         self.function = IdMap[self.functionId]
-        self.values = IdMap[self.valuesId]
+        self.values = []
+        self.impossible_values = []
+        if IdMap[self.valuesId]:
+            for v in IdMap[self.valuesId]:
+                if v.isImpossible():
+                    self.impossible_values.append(v)
+                else:
+                    self.values.append(v)
+                v.setId(IdMap)
         self.typeScope = IdMap[self.typeScopeId]
         self.astParent = IdMap[self.astParentId]
         self.astOperand1 = IdMap[self.astOperand1Id]
@@ -308,11 +419,28 @@ class Token:
                 return value
         return None
 
+    def getKnownIntValue(self):
+        """
+        If token has a known int value then return that.
+        Otherwise returns None
+        """
+        if not self.values:
+            return None
+        for value in self.values:
+            if value.valueKind == 'known':
+                return value.intvalue
+        return None
+
+    def isUnaryOp(self, op):
+        return self.astOperand1 and (self.astOperand2 is None) and self.str == op
+
+    def isBinaryOp(self):
+        return self.astOperand1 and self.astOperand2
 
 class Scope:
     """
     Scope. Information about global scope, function scopes, class scopes, inner scopes, etc.
-    C++ class: http://cppcheck.net/devinfo/doxyoutput/classScope.html
+    C++ class: https://cppcheck.sourceforge.io/devinfo/doxyoutput/classScope.html
 
     Attributes
         bodyStart      The { Token for this scope
@@ -337,6 +465,8 @@ class Scope:
     nestedIn = None
     type = None
     isExecutable = None
+    varlistId = None
+    varlist = None
 
     def __init__(self, element):
         self.Id = element.get('id')
@@ -352,6 +482,8 @@ class Scope:
         self.type = element.get('type')
         self.isExecutable = (self.type in ('Function', 'If', 'Else', 'For', 'While', 'Do',
                                            'Switch', 'Try', 'Catch', 'Unconditional', 'Lambda'))
+        self.varlistId = list()
+        self.varlist = list()
 
     def __repr__(self):
         attrs = ["Id", "className", "functionId", "bodyStartId", "bodyEndId",
@@ -366,53 +498,61 @@ class Scope:
         self.bodyEnd = IdMap[self.bodyEndId]
         self.nestedIn = IdMap[self.nestedInId]
         self.function = IdMap[self.functionId]
+        for v in self.varlistId:
+            value = IdMap.get(v)
+            if value:
+                self.varlist.append(value)
 
 
 class Function:
     """
     Information about a function
     C++ class:
-    http://cppcheck.net/devinfo/doxyoutput/classFunction.html
+    https://cppcheck.sourceforge.io/devinfo/doxyoutput/classFunction.html
 
     Attributes
         argument                Argument list
+        token                   Token in function implementation
         tokenDef                Token in function definition
         isVirtual               Is this function is virtual
         isImplicitlyVirtual     Is this function is virtual this in the base classes
-        isStatic                Is this function is static
+        isInlineKeyword         Is inline keyword used
+        isStatic                Is this function static?
     """
 
     Id = None
     argument = None
     argumentId = None
+    token = None
+    tokenId = None
     tokenDef = None
     tokenDefId = None
     name = None
     type = None
     isVirtual = None
     isImplicitlyVirtual = None
+    isInlineKeyword = None
     isStatic = None
     nestedIn = None
 
     def __init__(self, element, nestedIn):
         self.Id = element.get('id')
+        self.tokenId = element.get('token')
         self.tokenDefId = element.get('tokenDef')
         self.name = element.get('name')
         self.type = element.get('type')
-        isVirtual = element.get('isVirtual')
-        self.isVirtual = (isVirtual and isVirtual == 'true')
-        isImplicitlyVirtual = element.get('isImplicitlyVirtual')
-        self.isImplicitlyVirtual = (isImplicitlyVirtual and isImplicitlyVirtual == 'true')
-        isStatic = element.get('isStatic')
-        self.isStatic = (isStatic and isStatic == 'true')
+        self.isImplicitlyVirtual = element.get('isImplicitlyVirtual', 'false') == 'true'
+        self.isVirtual = element.get('isVirtual', 'false') == 'true'
+        self.isInlineKeyword = element.get('isInlineKeyword', 'false') == 'true'
+        self.isStatic = element.get('isStatic', 'false') == 'true'
         self.nestedIn = nestedIn
 
         self.argument = {}
         self.argumentId = {}
 
     def __repr__(self):
-        attrs = ["Id", "tokenDefId", "name", "type", "isVirtual",
-                 "isImplicitlyVirtual", "isStatic", "argumentId"]
+        attrs = ["Id", "tokenId", "tokenDefId", "name", "type", "isVirtual",
+                 "isImplicitlyVirtual", "isInlineKeyword", "isStatic", "argumentId"]
         return "{}({})".format(
             "Function",
             ", ".join(("{}={}".format(a, repr(getattr(self, a))) for a in attrs))
@@ -421,6 +561,7 @@ class Function:
     def setId(self, IdMap):
         for argnr, argid in self.argumentId.items():
             self.argument[argnr] = IdMap[argid]
+        self.token = IdMap.get(self.tokenId, None)
         self.tokenDef = IdMap[self.tokenDefId]
 
 
@@ -428,7 +569,7 @@ class Variable:
     """
     Information about a variable
     C++ class:
-    http://cppcheck.net/devinfo/doxyoutput/classVariable.html
+    https://cppcheck.sourceforge.io/devinfo/doxyoutput/classVariable.html
 
     Attributes:
         nameToken       Name token in variable declaration
@@ -482,19 +623,17 @@ class Variable:
         self.access = element.get('access')
         self.scopeId = element.get('scope')
         self.scope = None
-        self.isArgument = element.get('isArgument') == 'true'
+        self.isArgument = (self.access and self.access == 'Argument')
         self.isArray = element.get('isArray') == 'true'
         self.isClass = element.get('isClass') == 'true'
         self.isConst = element.get('isConst') == 'true'
-        self.isGlobal = element.get('access') == 'Global'
+        self.isGlobal = (self.access and self.access == 'Global')
         self.isExtern = element.get('isExtern') == 'true'
-        self.isLocal = element.get('isLocal') == 'true'
+        self.isLocal = (self.access and self.access == 'Local')
         self.isPointer = element.get('isPointer') == 'true'
         self.isReference = element.get('isReference') == 'true'
         self.isStatic = element.get('isStatic') == 'true'
-        self.constness = element.get('constness')
-        if self.constness:
-            self.constness = int(self.constness)
+        self.constness = int(element.get('constness',0))
 
     def __repr__(self):
         attrs = ["Id", "nameTokenId", "typeStartTokenId", "typeEndTokenId",
@@ -513,6 +652,21 @@ class Variable:
         self.scope = IdMap[self.scopeId]
 
 
+class TypedefInfo:
+    """
+    TypedefInfo class -- information about typedefs
+    """
+    name = None
+    used = None
+    file = None
+    linenr = None
+    column = None
+
+    def __init__(self, element):
+        self.name = element.get('name')
+        _load_location(self, element)
+        self.used = (element.get('used') == '1')
+
 class Value:
     """
     Value class
@@ -541,23 +695,42 @@ class Value:
     def isPossible(self):
         return self.valueKind and self.valueKind == 'possible'
 
+    def isImpossible(self):
+        return self.valueKind and self.valueKind == 'impossible'
+
     def __init__(self, element):
         self.intvalue = element.get('intvalue')
         if self.intvalue:
             self.intvalue = int(self.intvalue)
-        self.tokvalue = element.get('tokvalue')
+        self._tokvalueId = element.get('tokvalue')
         self.floatvalue = element.get('floatvalue')
         self.containerSize = element.get('container-size')
+        self.iteratorStart = element.get('iterator-start')
+        self.iteratorEnd = element.get('iterator-end')
+        self._lifetimeId = element.get('lifetime')
+        self.lifetimeScope = element.get('lifetime-scope')
+        self.lifetimeKind = element.get('lifetime-kind')
+        self._symbolicId = element.get('symbolic')
+        self.symbolicDelta = element.get('symbolic-delta')
         self.condition = element.get('condition-line')
+        self.bound = element.get('bound')
+        self.path = element.get('path')
         if self.condition:
             self.condition = int(self.condition)
         if element.get('known'):
             self.valueKind = 'known'
         elif element.get('possible'):
             self.valueKind = 'possible'
+        elif element.get('impossible'):
+            self.valueKind = 'impossible'
         if element.get('inconclusive'):
             self.inconclusive = True
 
+    def setId(self, IdMap):
+        self.tokvalue = IdMap.get(self._tokvalueId)
+        self.lifetime = IdMap.get(self._lifetimeId)
+        self.symbolic = IdMap.get(self._symbolicId)
+
     def __repr__(self):
         attrs = ["intvalue", "tokvalue", "floatvalue", "containerSize",
                     "condition", "valueKind", "inconclusive"]
@@ -573,7 +746,7 @@ class ValueFlow:
     Each possible value has a ValueFlow::Value item.
     Each ValueFlow::Value either has a intvalue or tokvalue
     C++ class:
-    http://cppcheck.net/devinfo/doxyoutput/classValueFlow_1_1Value.html
+    https://cppcheck.sourceforge.io/devinfo/doxyoutput/classValueFlow_1_1Value.html
 
     Attributes:
         values    Possible values
@@ -626,12 +799,11 @@ class Suppression:
 
     def isMatch(self, file, line, message, errorId):
         if ((self.fileName is None or fnmatch(file, self.fileName))
-                and (self.lineNumber is None or line == self.lineNumber)
+                and (self.lineNumber is None or int(line) == int(self.lineNumber))
                 and (self.symbolName is None or fnmatch(message, '*'+self.symbolName+'*'))
                 and fnmatch(errorId, self.errorId)):
             return True
-        else:
-            return False
+        return False
 
 
 class Configuration:
@@ -643,6 +815,8 @@ class Configuration:
     Attributes:
         name          Name of the configuration, "" for default
         directives    List of Directive items
+        macro_usage   List of used macros
+        preprocessor_if_conditions  List of preprocessor if conditions that was evaluated during preprocessing
         tokenlist     List of Token items
         scopes        List of Scope items
         functions     List of Function items
@@ -653,22 +827,30 @@ class Configuration:
 
     name = ''
     directives = []
+    macro_usage = []
+    preprocessor_if_conditions = []
     tokenlist = []
     scopes = []
     functions = []
     variables = []
+    typedefInfo = []
     valueflow = []
     standards = None
+    clang_warnings = []
 
     def __init__(self, name):
         self.name = name
         self.directives = []
+        self.macro_usage = []
+        self.preprocessor_if_conditions = []
         self.tokenlist = []
         self.scopes = []
         self.functions = []
         self.variables = []
+        self.typedefInfo = []
         self.valueflow = []
         self.standards = Standards()
+        self.clang_warnings = []
 
     def set_tokens_links(self):
         """Set next/previous links between tokens."""
@@ -680,7 +862,7 @@ class Configuration:
             prev = token
 
     def set_id_map(self, arguments):
-        IdMap = {None: None, '0': None, '00000000': None, '0000000000000000': None}
+        IdMap = {None: None, '0': None, '00000000': None, '0000000000000000': None, '0x0': None}
         for token in self.tokenlist:
             IdMap[token.Id] = token
         for scope in self.scopes:
@@ -794,6 +976,7 @@ class CppcheckData:
         filename          Path to Cppcheck dump file
         rawTokens         List of rawToken elements
         suppressions      List of Suppressions
+        files             Source files for elements occurred in this configuration
 
     To iterate through all configurations use such code:
     @code
@@ -823,17 +1006,16 @@ class CppcheckData:
     @endcode
     """
 
-    rawTokens = []
-    platform = None
-    suppressions = []
-
     def __init__(self, filename):
         """
         :param filename: Path to Cppcheck dump file
         """
         self.filename = filename
+        self.rawTokens = []
+        self.platform = None
+        self.suppressions = []
+        self.files = []
 
-        files = []  # source files for elements occurred in this configuration
         platform_done = False
         rawtokens_done = False
         suppressions_done = False
@@ -850,10 +1032,10 @@ class CppcheckData:
             elif node.tag == 'rawtokens' and event == 'end':
                 for rawtokens_node in node:
                     if rawtokens_node.tag == 'file':
-                        files.append(rawtokens_node.get('name'))
+                        self.files.append(rawtokens_node.get('name'))
                     elif rawtokens_node.tag == 'tok':
                         tok = Token(rawtokens_node)
-                        tok.file = files[int(rawtokens_node.get('fileIndex'))]
+                        tok.file = self.files[int(rawtokens_node.get('fileIndex'))]
                         self.rawTokens.append(tok)
                 rawtokens_done = True
             elif node.tag == 'suppressions' and event == 'end':
@@ -861,6 +1043,9 @@ class CppcheckData:
                     self.suppressions.append(Suppression(suppressions_node))
                 suppressions_done = True
 
+        global current_dumpfile_suppressions
+        current_dumpfile_suppressions = self.suppressions
+
         # Set links between rawTokens.
         for i in range(len(self.rawTokens)-1):
             self.rawTokens[i+1].previous = self.rawTokens[i]
@@ -883,13 +1068,11 @@ class CppcheckData:
         cfg_function = None
         cfg_valueflow = None
 
-        # Scopes contains <varlist> with all occurred variables. Some of them
-        # appearaed in <variables> node for this configuration.
-        # Others are arguments of functions.
-        # They have similar tag <var> but doesn't contain any attributes. So we
-        # set set a special state when iterate <varlist> node to prevent
-        # overriding of cfg.variables list with empty values.
-        iter_varlist = False
+        # Iterating <varlist> in a <scope>.
+        iter_scope_varlist = False
+
+        # Iterating <typedef-info>
+        iter_typedef_info = False
 
         # Use iterable objects to traverse XML tree for dump files incrementally.
         # Iterative approach is required to avoid large memory consumption.
@@ -906,6 +1089,12 @@ class CppcheckData:
                     cfg = None
                     cfg_arguments = []
 
+            elif node.tag == 'clang-warning' and event == 'start':
+                cfg.clang_warnings.append({'file': node.get('file'),
+                                           'line': int(node.get('line')),
+                                           'column': int(node.get('column')),
+                                           'message': node.get('message')})
+
             # Parse standards
             elif node.tag == "standards" and event == 'start':
                 continue
@@ -920,6 +1109,14 @@ class CppcheckData:
             elif node.tag == 'directive' and event == 'start':
                 cfg.directives.append(Directive(node))
 
+            # Parse macro usage
+            elif node.tag == 'macro' and event == 'start':
+                cfg.macro_usage.append(MacroUsage(node))
+
+            # Preprocessor #if/#elif condition
+            elif node.tag == "if-cond" and event == 'start':
+                cfg.preprocessor_if_conditions.append(PreprocessorIfCondition(node))
+
             # Parse tokens
             elif node.tag == 'tokenlist' and event == 'start':
                 continue
@@ -933,9 +1130,9 @@ class CppcheckData:
                 cfg.scopes.append(Scope(node))
             elif node.tag == 'varlist':
                 if event == 'start':
-                    iter_varlist = True
+                    iter_scope_varlist = True
                 elif event == 'end':
-                    iter_varlist = False
+                    iter_scope_varlist = False
 
             # Parse functions
             elif node.tag == 'functionList' and event == 'start':
@@ -956,11 +1153,20 @@ class CppcheckData:
 
             # Parse variables
             elif node.tag == 'var' and event == 'start':
-                var = Variable(node)
-                if var.nameTokenId:
-                    cfg.variables.append(var)
-                elif not iter_varlist:
-                    cfg_arguments.append(var)
+                if iter_scope_varlist:
+                    cfg.scopes[-1].varlistId.append(node.get('id'))
+                else:
+                    var = Variable(node)
+                    if var.nameTokenId:
+                        cfg.variables.append(var)
+                    else:
+                        cfg_arguments.append(var)
+
+            # Parse typedef info
+            elif node.tag == 'typedef-info':
+                iter_typedef_info = (event == 'start')
+            elif iter_typedef_info and node.tag == 'info' and event == 'start':
+                cfg.typedefInfo.append(TypedefInfo(node))
 
             # Parse valueflows (list of values)
             elif node.tag == 'valueflow' and event == 'start':
@@ -1072,12 +1278,32 @@ def ArgumentParser():
     parser.add_argument("--cli",
                         help="Addon is executed from Cppcheck",
                         action="store_true")
+    parser.add_argument("--file-list", metavar='<text>',
+                        default=None,
+                        help="file list in a text file")
     parser.add_argument("-q", "--quiet",
                         help='do not print "Checking ..." lines',
                         action="store_true")
     return parser
 
 
+def get_files(args):
+    """Return dump_files, ctu_info_files"""
+    all_files = args.dumpfile
+    if args.file_list:
+        with open(args.file_list, 'rt') as f:
+            for line in f.readlines():
+                all_files.append(line.rstrip())
+    dump_files = []
+    ctu_info_files = []
+    for f in all_files:
+        if f.endswith('.ctu-info'):
+            ctu_info_files.append(f)
+        else:
+            dump_files.append(f)
+    return dump_files, ctu_info_files
+
+
 def simpleMatch(token, pattern):
     for p in pattern.split(' '):
         if not token or token.str != p:
@@ -1085,6 +1311,44 @@ def simpleMatch(token, pattern):
         token = token.next
     return True
 
+def get_function_call_name_args(token):
+    """Get function name and arguments for function call
+    name, args = get_function_call_name_args(tok)
+    """
+    if token is None:
+        return None, None
+    if not token.isName or not token.scope.isExecutable:
+        return None, None
+    if not simpleMatch(token.next, '('):
+        return None, None
+    if token.function:
+        nametok = token.function.token
+        if nametok is None:
+            nametok = token.function.tokenDef
+        if token in (token.function.token, token.function.tokenDef):
+            return None, None
+        name = nametok.str
+        while nametok.previous and nametok.previous.previous and nametok.previous.str == '::' and nametok.previous.previous.isName:
+            name = nametok.previous.previous.str + '::' + name
+            nametok = nametok.previous.previous
+        scope = token.function.nestedIn
+        while scope:
+            if scope.className:
+                name = scope.className + '::' + name
+            scope = scope.nestedIn
+    else:
+        nametok = token
+        name = nametok.str
+        while nametok.previous and nametok.previous.previous and nametok.previous.str == '::' and nametok.previous.previous.isName:
+            name = nametok.previous.previous.str + '::' + name
+            nametok = nametok.previous.previous
+    return name, getArguments(token)
+
+def is_suppressed(location, message, errorId):
+    for suppression in current_dumpfile_suppressions:
+        if suppression.isMatch(location.file, location.linenr, message, errorId):
+            return True
+    return False
 
 def reportError(location, severity, message, addon, errorId, extra=''):
     if '--cli' in sys.argv:
@@ -1098,7 +1362,39 @@ def reportError(location, severity, message, addon, errorId, extra=''):
                 'extra': extra}
         sys.stdout.write(json.dumps(msg) + '\n')
     else:
+        if is_suppressed(location, message, '%s-%s' % (addon, errorId)):
+            return
         loc = '[%s:%i]' % (location.file, location.linenr)
         if len(extra) > 0:
             message += ' (' + extra + ')'
         sys.stderr.write('%s (%s) %s [%s-%s]\n' % (loc, severity, message, addon, errorId))
+        global EXIT_CODE
+        EXIT_CODE = 1
+
+def reportSummary(dumpfile, summary_type, summary_data):
+    # dumpfile ends with ".dump"
+    ctu_info_file = dumpfile[:-4] + "ctu-info"
+    with open(ctu_info_file, 'at') as f:
+        msg = {'summary': summary_type, 'data': summary_data}
+        f.write(json.dumps(msg) + '\n')
+
+
+def get_path_premium_addon():
+    p = pathlib.Path(sys.argv[0]).parent.parent
+
+    for ext in ('.exe', ''):
+        p1 = os.path.join(p, 'premiumaddon' + ext)
+        p2 = os.path.join(p, 'cppcheck' + ext)
+        if os.path.isfile(p1) and os.path.isfile(p2):
+            return p1
+    return None
+
+
+def cmd_output(cmd):
+    with subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE) as p:
+        comm = p.communicate()
+        out = comm[0]
+        if p.returncode == 1 and len(comm[1]) > 2:
+            out = comm[1]
+        return out.decode(encoding='utf-8', errors='ignore')
+
